@@ -1,6 +1,6 @@
 /*
  * Worldvisions Weaver Software:
- *   Copyright (C) 1997-2004 Net Integration Technologies, Inc.
+ *   Copyright (C) 1997-2005 Net Integration Technologies, Inc.
  *
  * Avery's insanely fast alternative to Fetchmail
  *
@@ -20,6 +20,7 @@
 #include "wvlogrcv.h"
 #include "wvsslstream.h"
 #include "wvhashtable.h"
+#include "wvargs.h"
 #include "wvcrash.h"
 #include "wvhex.h"
 
@@ -35,18 +36,12 @@
 struct LogNum
 {
     int num;
-    const WvLog *src;
+    WvString src;
 };
 
-// cheesy hash function to do the job, basically
-unsigned int WvHash(const WvLog *x)
-{
-    return WvHash((int)x);
-}
+DeclareWvDict(LogNum, WvString, src);
 
 
-DeclareWvDict(LogNum, const WvLog *, src);
-
 class RetchLog : public WvLogConsole
 {
 public:
@@ -64,7 +59,7 @@ public:
 
 void RetchLog::_begin_line()
 {
-    if (!strncmp(last_source->app, "PopRetriever ", 13))
+    if (!strncmp(last_source, "PopRetriever ", 13))
     {
 	LogNum *lognum = lognums[last_source];
 	if (!lognum)
@@ -75,7 +70,7 @@ void RetchLog::_begin_line()
 	    lognums.add(lognum, true);
 	}
 	
-	// identify the connectionw without being too verbose
+	// identify the connection without being too verbose
 	if (lognum->num < 26)
 	{
 	    char str[2];
@@ -109,7 +104,8 @@ void signal_handler(int signum)
 static WvPopClient *newpop(WvStringParm acct,
 			   WvStringParm _pass, WvStringParm _deliverto,
                            WvStringParm _mda, bool flush, bool apop_en,
-                           bool apop_fall_en, bool explode)
+                           bool apop_fall_en, bool explode, bool safemode,
+			   bool ignorerp)
 
 {
     WvString user(acct), serv, pass(_pass), deliverto(_deliverto),
@@ -140,38 +136,31 @@ static WvPopClient *newpop(WvStringParm acct,
     if (ssl) // FIXME: ssl verify should probably be set to something.
 	conn = new WvSSLStream(tcp, NULL);
     
-    return new WvPopClient(conn, acct, pass, deliverto, mda, flush, apop_en, apop_fall_en, explode);
+    return new WvPopClient(conn, acct, pass, deliverto, mda, flush, apop_en, apop_fall_en, explode, safemode, ignorerp);
 }
 
 
-static void usage(char *argv0, WvStringParm deliverto)
+static bool dec_log_level_cb(void *userdata)
 {
-    wvcon->print("Usage: %s [-d] [-dd] [-q] [-qq] [-V] [-F] [-c moniker ] "
-		 "[-t deliverto] [acct...]\n"
-		 "     -d   Print debug messages\n"
-		 "     -dd  Print lots of debug messages\n"
-		 "     -q   Quieter: don't print every message header\n"
-		 "     -qq  Way quieter: only print errors\n"
-		 "     -V   Print version and exit\n"
-		 "     -c   Use <moniker> instead of "
-		 "ini:~/.retchmail/retchmail\n"
-		 "     -F   Flush (delete) messages after downloading\n"
-	         "     -E   Send mail to the user on the system corresponding to the user the mail is sent to\n"
-		 "     -t   Deliver mail to <deliverto> (default '%s')\n"
-		 "  acct... list of email accounts (username@host) to "
-		 "retrieve from\n",
-		 argv0, (const char *)deliverto);
-    exit(1);
+    WvLog::LogLevel level = *static_cast<WvLog::LogLevel *>(userdata);
+    if ((int)level > (int)WvLog::Critical)
+	level = (WvLog::LogLevel)((int)level - 1);
+    return true;
 }
 
-extern char *optarg;
-extern int optind;
+
+static bool inc_log_level_cb(void *userdata)
+{
+    WvLog::LogLevel level = *static_cast<WvLog::LogLevel *>(userdata);
+    if ((int)level < (int)WvLog::Debug5)
+	level = (WvLog::LogLevel)((int)level + 1);
+    return true;
+}
 
 
 int main(int argc, char **argv)
 {
     bool flush = false, explode = false;
-    int c, count;
     WvLog::LogLevel lvl = WvLog::Debug1;
     WvString deliverto("");
     WvString confmoniker("");
@@ -188,45 +177,39 @@ int main(int argc, char **argv)
     if (pw)
 	deliverto = pw->pw_name;
 
-    while ((c = getopt(argc, argv, "dqVFEt:c:h?")) >= 0)
-    {
-	switch (c)
-	{
-	case 'd':
-	    if (lvl <= WvLog::Debug1)
-		lvl = WvLog::Debug2;
-	    else
-		lvl = WvLog::Debug5;
-	    break;
-	case 'q':
-	    if (lvl >= WvLog::Debug1)
-		lvl = WvLog::Info;
-	    else
-		lvl = WvLog::Notice;
-	    break;
-	case 'V':
-	    wvcon->print("Retchmail version %s\n", RETCHMAIL_VER_STRING);
-	    return 2;
-	case 'F':
-	    flush = true;
-	    break;
-	case 't':
-	    deliverto = optarg;
-	    break;
-	case 'c':
-	    confmoniker = optarg;
-	    break;
-	case 'E':
-	  explode = true;
-	  break;
-	case 'h':
-	case '?':
-	default:
-	    usage(argv[0], deliverto);
-	    break;
-	}
-    }
+    WvArgs args;
 
+    args.add_option('q', "quiet",
+		    "Decrease log level (can be used multiple times)",
+		    dec_log_level_cb, &lvl);
+    
+    args.add_option('d', "debug",
+		    "Increase log level (can be used multiple times)",
+		    inc_log_level_cb, &lvl);
+
+    args.add_set_bool_option('F', "flush", 
+			     "Flush (delete) messages after downloading",
+			     flush);
+
+    args.add_set_bool_option('E', "explode", 
+			     "Send mail to the user on the system corresponding to the user the mail is sent to",
+			     explode);
+    
+    args.add_option('t', "to", 
+		    WvString("Deliver mail to <deliverto> (default '%s')", deliverto),
+		    "deliverto", deliverto);
+
+    args.add_option('c', "config",
+		    "Use <moniker> instead of ini:~/.retchmail/retchmail",
+		    "moniker", confmoniker);
+    
+    args.set_version("Retchmail version " RETCHMAIL_VER_STRING);
+    args.set_email("<wvstreams-dev@lists.nit.ca>");
+    
+    WvStringList arguments;
+    if (!args.process(argc, argv, &arguments))
+	return 255;
+    
     WvString lockname("/tmp/retchmail.%s.pid", getlogin());
     WvLockFile lockfile(lockname);
 
@@ -248,7 +231,6 @@ int main(int argc, char **argv)
 	return 3;
     }
     
-    
     RetchLog logrcv(lvl);
     
     if (!confmoniker)
@@ -267,11 +249,13 @@ int main(int argc, char **argv)
 	exit(1);
     }
 
-    WvPopClient *cli;
+    WvPopClient *cli = NULL;
     bool apop_enable = cfg["retchmail"]["Enable APOP"].getmeint(0);
     bool apop_enable_fallback = cfg["retchmail"]["Enable APOP Fallback"].getmeint(0);
+    bool safemode = cfg["retchmail"]["Safe Mode"].getmeint(0);
+    bool ignorerp = cfg["retchmail"]["Ignore Return-Path"].getmeint(0);
   
-    if (optind == argc)	    
+    if (arguments.isempty())	    
     {
         UniConf sect = cfg["POP Servers"];
 	if (sect.haschildren())
@@ -284,7 +268,8 @@ int main(int argc, char **argv)
                              cfg["MDA Override"][i->key()].getme(
 				     "/usr/sbin/sendmail"),
                              flush, apop_enable, apop_enable_fallback,
-			     explode ? : cfg["Explode"][i->key()].getmeint(0));
+			     explode ? : cfg["Explode"][i->key()].getmeint(0),
+			     safemode, ignorerp);
 		WvIStreamList::globallist.append(cli, true, "client");
 	    }
 	}
@@ -292,30 +277,32 @@ int main(int argc, char **argv)
 	{
 	    fprintf(stderr, "\n-- No config file and no accounts given on the "
 		    "command line!\n");
-	    usage(argv[0], deliverto);
+	    args.print_usage(argc, argv);
 	}
     }
     else
     {
-	for (count = optind; count < argc; count++)
+	while (!arguments.isempty());
 	{
-	    WvString pass = cfg["POP Servers"][argv[count]].getme();
+	    WvString user(arguments.popstr());
+	    WvString pass = cfg["POP Servers"][user].getme();
 	    if (!pass)
 	    {
-		wvcon->print("Password for <%s>: ", argv[count]);
+		wvcon->print("Password for <%s>: ", user);
 		
 		system("stty -echo 2>/dev/null");
-		pass = wvcon->getline(-1);
+		pass = wvcon->blocking_getline(-1);
 		system("stty echo 2>/dev/null");
 		
 		wvcon->print("\n");
 	    }
 	    
-	    cli = newpop(argv[count], pass,
-			 cfg["POP Targets"][argv[count]].getme(deliverto),
-			 cfg["MDA Override"][argv[count]].getme(
+	    cli = newpop(user, pass,
+			 cfg["POP Targets"][user].getme(deliverto),
+			 cfg["MDA Override"][user].getme(
 				 "/usr/sbin/sendmail"),
-			 flush, apop_enable, apop_enable_fallback, explode);
+			 flush, apop_enable, apop_enable_fallback, explode,
+			 safemode, ignorerp);
 	    WvIStreamList::globallist.append(cli, true, "client");
 	}
     }
